import Crypto.Cipher.AES
import secrets
import hashlib
from typing import Tuple

"""
This is the file for all cryptographical implementations for the client.
Please use this file instead of other implementations.
The different cryptographical algorithms - for example AES - are stored in classes. It isn't intended to be seen as a class, but as a namespace
"""


"""
AES
We are using AES for most of the encryption.
We are using AES-OFB as default mode. Please don't choose another if you aren't absoloutely sure what you are doing.
"""

class AES:
    """
    encrypt is the standard function to encrypt a message with AES.
    The message and the key both have to be given in bytes.
    We will not use the key itself for the encryption, but its sha256 hash.
    The Initialization Vector will be generated with cryptographically secure pseudorandom
    The function will return a tuple of 2 byte-arrays. The first one is the ciphertext, the second one the Initialization Vector (IV)
    """
    @staticmethod
    def encrypt(msg: bytes, key: bytes, mode=Crypto.Cipher.AES.MODE_OFB) -> Tuple[bytes, bytes]:
        IV = secrets.token_bytes(16)
        hash = hashlib.sha256(key).digest()
        cipher = Crypto.Cipher.AES.new(key=hash, mode=mode, IV=IV)
        return cipher.encrypt(msg), IV
    
    """
    decrypt is the standard function to decrypt a message with AES.
    The message, the key and the Initialization Vector all have to be given in bytes. The message means here the ciphertext.
    We will not use the key itself for the decryption, but its sha256 hash.
    The function will return the encrypted message as str
    """
    @staticmethod
    def decrypt(msg, key, IV, mode=Crypto.Cipher.AES.MODE_OFB) -> bytes:
        hash = hashlib.sha256(key).digest()
        cipher = Crypto.Cipher.AES.new(key=hash, mode=mode, IV=IV)
        return cipher.decrypt(msg)